#!/bin/sh
#
# File:         client_config_fp.sh
#
# Product:      Tivoli Module
# Description:  Shell Script
# Author:               Module Generator
#
# Generated:    
#
# (C) COPYRIGHT Tivoli Systems, Inc., 1997
# All Rights Reserved
# Licensed Material - Property of Tivoli Systems, Inc.
#

###
#
#   errecho
#
errecho() {
  echo "$*" >&2
}

###
#
# is_lcf_ep
#
is_lcf_ep() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if [ -z "$ENDPOINT_OID" ]; then
    return 1
  elif echo $ENDPOINT_OID | awk -F\# '{print $1}' | grep "+$" > /dev/null; then
    # found a "+" in the oid, must be a TMA endpoint!
    return 0
  else
    return 1
  fi
}

########################################################################

if [ -n "" ]; then
        DEBUG=true
fi
[ "$DEBUG" = true ] && set -x

TASK_PROGPATH=`echo $0 | sed -e 's.\\\\./.g'`
THIS_SCRIPT="client_config_fp.sh"

#
#       Make sure we're running on a Managed Node
#
if is_lcf_ep; then
  errecho "Cannot run $THIS_SCRIPT: endpoint $ENDPOINT_OID is not a ManagedNode"
  exit 1
fi

#
#       The standard directory and the /Plus override script
#
ETC=/etc/Tivoli
SCRIPT=PLUSsetup_env.sh

#
#       If this is NT, then etc is in a different location
#
if [ x\"$OS\" = x\"Windows_NT\" ] ; then
     ETC=$SystemRoot/system32/drivers/etc/Tivoli
fi

#
#       Look for our override script, if not there, then use the
#               standard one.
#
if [ ! -f $ETC/$SCRIPT ]; then
        SCRIPT=setup_env.sh
fi

. $ETC/$SCRIPT

#
#	Set up some popular environment variables.
#
OSERV=`objcall 0.0.0 get_oserv`
export OSERV
INST_DIR=`objcall $OSERV query install_dir`
INST_DIR=`echo $INST_DIR | tr "\\\\\\\\" "/"`
export INST_DIR
REGION=`objcall $OSERV query whoami | awk '{print $1}'`

LIBRARY=`wlookup Library`

#
#	Get the region name to avoid collisions
#
IRO=`wlookup InterRegion`
IRONAME=`idlattr -t -g $IRO name string`
IRONAME=`eval echo $IRONAME`

DEFSEN=\"TivoliSentryDefaults#$IRONAME\"

MNOID=`objcall 0.0.0 get_host_location`
eval MNNAME=`idlattr -t -g $MNOID label string`
export MNNAME

PLUSDIR=$INST_DIR/generic_unix/TME/PLUS
LINKDIR=$PLUSDIR/LINK
PRODDIR=$PLUSDIR/TWS
PATH=$LINKDIR:$PRODDIR:$PATH
export PATH

cd $PRODDIR

. ./PLUSproduct-info.sh
export WINSTRUCTED
export MODULE_VERSION

#
#	Debugging code: by creating a directory called .plusdebug in
#		the appropriate TEMP directory, the output of scripts is
#		redirected to a file in TEMP by the same name as the script.
#
TEMP=/tmp
if [ x"$OS" = x"Windows_NT" ]; then
	TEMP=$DBDIR/tmp
else
	TEMP=/tmp
fi

DEBUGDIR=$TEMP/.plusdebug
[ -d "$DEBUGDIR" ] \
  && echo "*** Results redirected to $DEBUGDIR/$THIS_SCRIPT" \
  && exec > $DEBUGDIR/$THIS_SCRIPT 2>&1 && set -xv
[ -f "$DEBUGDIR" ] && set -xv
if [ -d "$DEBUGDIR" -o -f "$DEBUGDIR" -o "$DEBUG" = true ]; then
  DEBUG=true
else
  DEBUG=false
fi
export DEBUG

#
#	Messages
#
ERRCONFIG="Error configuring FilePackage"
SUCCESSCONFIG="Successfully configured FilePackage"
ERRNOARG="Error: Missing value for argument"
ERRSRCHOST="Error: Cannot find managed node"
ERRNOSRCHOST="Error: Must provide a ManagedNode name source host"
ERRSCRATCH="Error: Must provide a path to a scratch directory"
ERRMEDIAPATH="Error: Must provide a media path to device or TWS tar file"
ERRNOARCH="Error: Must select an Interpreter Type: Built-in or Other"
ERRSRCPATH="Must provide a source path to an existing directory"
ERRDESTPATH="Must provide destination directory"
ERRMAESTROUSER="Must provide TWS User name"
ERRMAESTROUID="Must provide a UNIX User ID for the TWS user"
ERRMAESTROGID="Must provide a UNIX Group ID for the TWS user"
ERRCPUMAP="Must provide a path to the TWS CPU map file on the Tivoli server"
ERRUNKNOWNFP="Error: Could not determine filepack id for this task"
ERRTARNOTSUPPORTED="The \"Tar-File or Device\" setting for the \"Get TWS Files from:\" option is not supported when the TWS sources reside on a Windows NT Managed Node."

###################
# Support Functions
###################

errexit()
{
	echo "$*" >&2
	exit 1
}

errmsg()
{
	echo "$*" >&2
}

######################################################################
#
#	fp_replace_keyword
#
fp_replace_keyword() {
  keyvalpair="$1"
  fpdesc="$2"
  tmpdesc=${fpdesc}.tmp

  # split the keyword and value
  key=`echo $keyvalpair | awk -F= '{print $1}'`
  sed -e "s~^${key}=.*~${keyvalpair}~" $fpdesc > $tmpdesc
  if grep "^${key}=" $tmpdesc > /dev/null ; then
    :  #it got there
  else
    echo "$keyvalpair" >> $tmpdesc
  fi
  mv $tmpdesc $fpdesc
}

######################################################################
#
#	fp_platform
#
fp_platform() {
  case "$1" in
    WINNT|nt)    fp_plat=nt;;
    WIN9*|win9*) fp_plat=win95;;
    WIN*|win*)   fp_plat=win;;
    NetWare|nw)  fp_plat=nw;;
    OS2|os2)     fp_plat=os2;;
    DOS|dos)     fp_plat=dos;;
    *?)          fp_plat=unix;;
  esac
  echo $fp_plat
}

######################################################################
#
#	lookup_filepack_id
#
lookup_filepack_id() {
  match_name=$1
  eval match_name="$match_name"
  for i in $FILEPACKS; do
    eval name='"$'${i}'"'
    eval uiname='"$'${i}'_UINAME"'
    if [ "$name" = "$match_name" ]; then
      echo "$i"
      return 0
    elif [ "$uiname" = "$match_name" ]; then
      echo "$i"
      return 0
    fi
  done
  return 1
}

######################################################################
#
#	replace_nested_fp_name
#
replace_nested_fp_name() {
  fpid="$1"
  fpdesc="$2"
  tmpdesc=${fpdesc}.tmp
  rm -f $tmpdesc

  eval fpname='"$'$fpid'"'
  eval fpnestfps='"$'$fpid'_FPNESTFPS"'

  if [ -f "$fpnestfps" ]; then
    # Copy the nested file package list, appending the inter-region name
    while read line; do
      if [ "$line" ]; then
        nestref=${line}
        if lookup_filepack_id "$nestref"; then
          nestid=`lookup_filepack_id "$nestref"`
          eval nestname='"$'$nestid'"'
          nestname=${nestname}#${IRONAME}
          sed -e "s~^${nestref}~\"${nestname}\"~" $fpdesc > $tmpdesc
        else
          errecho "Unable to locate nested file package!"
          return 1
        fi
      fi
    done <$fpnestfps
  fi
  if [ -f "$tmpdesc" ]; then
    mv $tmpdesc $fpdesc
  fi
}

###################
# Process Arguments
###################

#
#	Usage Check
#
if [ $# -ne 12 ] ; then
	errexit "Usage: $0 interp-type target_type custom-target src-host media src-path staging-area maestro_user dest-path maestro-uid maestro-gid cpu-map-path"
fi

#
#	set some local environment variables from the command line arguments
#
TARGET=$1
echo TARGET: $TARGET >&2
shift

OTHER=$1
echo OTHER: $OTHER >&2
shift

TARGET_TYPE=$1
echo TARGET_TYPE: $TARGET_TYPE >&2
shift


SRC_HOST=$1
echo SRC_HOST: $SRC_HOST >&2
shift

SRC_MEDIA=$1
echo SRC_MEDIA: $SRC_MEDIA >&2
shift

MEDIA_PATH=$1
MEDIA_PATH=`echo $MEDIA_PATH | tr "\\\\\\\\" "/"`
echo MEDIA_PATH: $MEDIA_PATH >&2
shift

SRC_STAGING=$1
SRC_STAGING=`echo $SRC_STAGING | tr "\\\\\\\\" "/"`
echo SRC_STAGING: $SRC_STAGING >&2
shift

MAESTRO_USER=$1
echo MAESTRO_USER: $MAESTRO_USER >&2
shift

DEST_PATH=$1
DEST_PATH=`echo $DEST_PATH | tr "\\\\\\\\" "/"`
echo DEST_PATH: $DEST_PATH >&2
shift

MAESTRO_UID=$1
echo MAESTRO_UID: $MAESTRO_UID >&2
shift

MAESTRO_GID=$1
echo MAESTRO_GID: $MAESTRO_GID >&2
shift

CPU_MAP=$1
CPU_MAP=`echo $CPU_MAP | tr "\\\\\\\\" "/"`
echo CPU_MAP: $CPU_MAP >&2

#
#	Check for Intepreter or alternate target
#
if [ x"$TARGET" = xother ] ;then
	if [ x"$OTHER" = x ] ;then
		echo $ERRNOARCH >&2
		exit 1
	fi
	TARGET=$OTHER
fi

if [ x"$TARGET_TYPE" = xendpoint ];then
	IS_ENDPOINT=true
else
	IS_ENDPOINT=false
fi

#
#	Set the custom environment
#
CUSTOM_ENV="TARGET='$TARGET' IS_ENDPOINT='$IS_ENDPOINT'"

#
#	Check for existing installation or tape
#
if [ x"$SRC_MEDIA" != x"existing" ] ;then
	#
	# get it from tape
	#
	GET_FROM_TAPE=1
	EMPTY_ARG=0

	if [ x"$MEDIA_PATH" = x ] ;then
		EMPTY_ARG=1
		echo $ERRMEDIAPATH >&2
	fi

	if [ x"$SRC_STAGING" = x ] ;then
		EMPTY_ARG=1
		echo "$ERRSCRATCH" >&2
	fi

	if [ $EMPTY_ARG -eq 1 ] ; then
                errexit "$ERRNOARG"
	fi

	SRC_PATH=$SRC_STAGING/$TARGET/maestro
#	CUSTOM_ENV="$CUSTOM_ENV SRC_TAR='$MEDIA_PATH' SRC_STAGING='$SRC_PATH'"
else
	if [ x"$MEDIA_PATH" = x ] ;then
		echo $ERRSRCPATH >&2
		exit 1
	fi	
	SRC_PATH=$MEDIA_PATH
fi


#Check for remaining empty arguments
EMPTY_ARG=0

#
#	Check for remaining empty arguments
#
if [ x"$SRC_HOST" = x ] ;then
	EMPTY_ARG=1
	echo $ERRNOSRCHOST >&2
fi

if [ x"$MAESTRO_USER" = x ] ;then
	EMPTY_ARG=1
	echo $ERRMAESTROUSER >&2
fi

if [ x"$DEST_PATH" = x ] ;then
	EMPTY_ARG=1
	echo $ERRDESTPATH >&2
fi

if [ x"$MAESTRO_UID" = x ] ;then
	EMPTY_ARG=1
	echo $ERRMAESTROUID >&2
fi

if [ x"$MAESTRO_GID" = x ] ;then
	EMPTY_ARG=1
	echo $ERRMAESTROGID >&2
fi

if [ x"$CPU_MAP" = x ] ;then
	EMPTY_ARG=1
	echo $ERRCPUMAP >&2
fi

if [ $EMPTY_ARG -eq 1 ] ; then
        errexit "$ERRNOARG"
fi

#
# Make sure it makes sense to continue, for example:
#
wlookup -r ManagedNode "$SRC_HOST" > /dev/null 2>&1
STATUS=$?
if [ $STATUS -ne 0 ] ;then
	errexit "$ERRSRCHOST $SRC_HOST"
fi

#
# Get source host OS type, check source file format, and set log directory
#
LOGDIR=/tmp
MN_ID=`wlookup -r ManagedNode "$SRC_HOST"`
MN_OID=`echo $MN_ID | awk -F# '{ print $1 }'`
MN_OSTYPE=`idlcall $MN_OID TMF_ManagedNode::SysInfo::os_name`
if [ x$MN_OSTYPE = x\"Windows_NT\" ] ; then

	if [ x"$GET_FROM_TAPE" = x1 ] ; then
		errexit $ERRTARNOTSUPPORTED 
	fi

        LOGDIR=/temp
fi

CUSTOM_ENV="$CUSTOM_ENV SPACE_NEEDED=150000 MAP_FILE='$CPU_MAP' MAESTRO_USER='$MAESTRO_USER' MAESTRO_DIR='$DEST_PATH' UID_VALUE='$MAESTRO_UID' GID_VALUE='$MAESTRO_GID'"
#CUSTOM_ENV="$CUSTOM_ENV DEST_PATH='$DEST_PATH' SPACE_NEEDED=150000 MAP_FILE='$CPU_MAP' MAESTRO_USER='$MAESTRO_USER' MAESTRO_DIR='$DEST_PATH' UID_VALUE='$MAESTRO_UID' GID_VALUE='$MAESTRO_GID'"

#**********************************************************************
#	
#	Add code here to modify file package description file
#
#**********************************************************************

#
#	FP gets a value FILEPACK_<NUM>, which references the filepackage 
#	definition variables in PLUSproduct-info.sh
#

FP=
THIS_COMPONENT_NAME="Install UNIX TWS Clients"
for fp in $FILEPACKS; do
   if [ "$WINSTRUCTED" = true ]; then
	eval COMPNAME='"$'$fp'_UINAME"'    # UINAME reflects component name in winstruct-land
   else
	eval COMPNAME='"$'$fp'"'
   fi
   if [ "$COMPNAME" -a "$COMPNAME" = "$THIS_COMPONENT_NAME" ]; then     # it's us!
	FP="$fp"
	break
   fi
done
if [ ! "$FP" ]; then
	errexit "$ERRUNKNOWNFP"
fi


eval FPNAME='"$'$FP'"'
eval FPSHORTNAME='"$'$FP'_SHORTNAME"'
eval FPDEF='"$'$FP'_FPDEF"'
eval FPKEYWORDS='"$'$FP'_FPKEYWORDS"'
eval FPDEFHEADER='"$'$FP'_FPDEFHEADER"'
eval FPINCLFILES='"$'$FP'_FPINCLFILES"'
eval FPEXCLFILES='"$'$FP'_FPEXCLFILES"'
eval FPNESTFPS='"$'$FP'_FPNESTFPS"'

# These may have a nested variable, so eval them again:
eval FPDEF=$FPDEF
eval FPKEYWORDS=$FPKEYWORDS
eval FPDEFHEADER=$FPDEFHEADER
eval FPINCLFILES=$FPINCLFILES
eval FPEXCLFILES=$FPEXCLFILES
eval FPNESTFPS=$FPNESTFPS

#
#	These environment variables contain the install script names
#
eval FPBEFOREINSTALL='"$'$FP'_BEFORE_INSTALL"'
eval FPAFTERINSTALL='"$'$FP'_AFTER_INSTALL"'
eval FPONCOMMIT='"$'$FP'_ON_COMMIT"'
eval FPBEFOREREMOVAL='"$'$FP'_BEFORE_REMOVE"'
eval FPAFTERREMOVAL='"$'$FP'_AFTER_REMOVE"'
eval FPONERROR='"$'$FP'_ON_ERROR"'

    # These vars could contain nested vars, eval them again
    eval fpdef=$fpdef
    eval fpkeywords=$fpkeywords
    eval fpdefheader=$fpdefheader
    eval fpinclfiles=$fpinclfiles
    eval fpexclfiles=$fpexclfiles
    eval fpnestfps=$fpnestfps

    keyword_suffixes="
      before_prog_path
      before_input_path
      after_prog_path
      after_input_path
      commit_prog_path
      commit_input_path
      removal_prog_path
      removal_input_path
      after_removal_prog_path
      after_removal_input_path
      on_error_prog_path
      on_error_input_path
    "
    eval before_prog_path='"$'$fp'_BEFORE_INSTALL"'
    eval before_input_path='"$'$fp'_BEFORE_INSTALL_INPUT"'
    eval after_prog_path='"$'$fp'_AFTER_INSTALL"'
    eval after_input_path='"$'$fp'_AFTER_INSTALL_INPUT"'
    eval commit_prog_path='"$'$fp'_ON_COMMIT"'
    eval commit_input_path='"$'$fp'_ON_COMMIT_INPUT"'
    eval removal_prog_path='"$'$fp'_BEFORE_REMOVE"'
    eval removal_input_path='"$'$fp'_BEFORE_REMOVE_INPUT"'
    eval after_removal_prog_path='"$'$fp'_AFTER_REMOVE"'
    eval after_removal_input_path='"$'$fp'_AFTER_REMOVE_INPUT"'
    eval on_error_prog_path='"$'$fp'_ON_ERROR"'
    eval on_error_input_path='"$'$fp'_ON_ERROR_INPUT"'

    ## plat=`fp_platform $fpplatform`
    plat=`fp_platform $TARGET`

FPDESC=$PRODDIR/$FPSHORTNAME.fp
NEWFPDESC=$TEMP/$FPSHORTNAME.fp

#
#	Here is where you may want to modify the filepackage description
#	before you import it into the file package profile.
#	In this example, we make a processed copy in $NEWFPDESC so that the 
#       original description file is intact for the next time the fp 
#       configuration task is run.
#
rm -f $NEWFPDESC

# Copy the filepackage definition header
cat $FPDEFHEADER >$NEWFPDESC

# Add the custom environment as constructed from this task's arg list
echo prog_env='"'$CUSTOM_ENV'"' >> $NEWFPDESC

#
#	The REPLACE_KEYWORD variable contains the names of other variables
#	that contain keyword value pairs that will be replaced in the key-
#	word list.  Add more keywords by adding more names to the
#	REPLACE_KEYWORDS variable.  The two commented lines below show
#	how to add another variable.
#

REPLACE_KEYWORD1="nested_first=y"
REPLACE_KEYWORD2="log_host=$SRC_HOST"
REPLACE_KEYWORD3="log_file=$LOGDIR/maestro_client_install.unix"
REPLACE_KEYWORD4="stop_on_error=n"
REPLACE_KEYWORD5="unix_platform_prefix=$DEST_PATH"
REPLACE_KEYWORD6="unix_before_skip_non_zero=y"
REPLACE_KEYWORDS="REPLACE_KEYWORD1 REPLACE_KEYWORD2 REPLACE_KEYWORD3 REPLACE_KEYWORD4 REPLACE_KEYWORD5 REPLACE_KEYWORD6"

cat $FPKEYWORDS >> $NEWFPDESC

# First replace keywords with values from component
for kw in $keyword_suffixes; do
      eval fp_replace_keyword ${plat}_${kw}='"$'$kw'"' $NEWFPDESC
done

# Now, using the local keyword list, replace values from the $FPKEYWORDS file
for repkey in $REPLACE_KEYWORDS; do
      eval KEYVALPAIR='"$'$repkey'"'
      eval fp_replace_keyword $KEYVALPAIR $NEWFPDESC
done

echo "%" >>$NEWFPDESC

if [ x"$GET_FROM_TAPE" = x1 ] ; then
        echo "\`PATH=/bin:/usr/bin:/usr/ucb && \
                export PATH && \
                mkdir -p $SRC_PATH && \
                cd  $SRC_PATH && \
                tar -xf $MEDIA_PATH \`" >> $NEWFPDESC
fi

# Copy the actual file list.
#cat $FPINCLFILES >>$NEWFPDESC
echo "$SRC_PATH/*" >>$NEWFPDESC
echo "%" >>$NEWFPDESC

# Copy the nested file package list
cat $FPNESTFPS >>$NEWFPDESC

echo "%" >>$NEWFPDESC

# Copy the excluded file list
if [ -f $FPEXCLFILES ]; then
	cat $FPEXCLFILES >>$NEWFPDESC
fi

# make sure nested fp's are referred to correctly 
#  (i.e. may need to use the winstructed name, 
#   and they need the IRONAME appended)
replace_nested_fp_name $fp $NEWFPDESC

#
# add cpu_map file location 
#
sed -e "s,CPUMAP_FILE,${CPU_MAP}," $PRODDIR/unix_ancillary_filelist > ${TEMP}/unix_ancillary_filelist.tmp

ANCILLARY_FPNAME="Ancillary Filepack for UNIX TWS"

wsetfpcontents -F @${MNNAME}:${TEMP}/unix_ancillary_filelist.tmp "@FilePackage:$ANCILLARY_FPNAME#$IRONAME"

wsetfpopts -T gen -m 0755 "@FilePackage:$ANCILLARY_FPNAME#$IRONAME"

rm -rf ${TEMP}/unix_ancillary_filelist.tmp

#
#	Now use the general purpose wcrtsprfl to create the file pack and plus link
#
if [ "$WINSTRUCTED" = "true" ]; then
	eval FPUINAME='"$'$FP'_UINAME"'
	eval FPPARENT='"$'$FP'_PARENT"'
else
	FPUINAME="$FPNAME"
	FPPARENT="$FPNAME"
fi
#if [ "$TARGET" ]; then
#   CONFIGURED_FPNAME="$FPUINAME for $TARGET"
#else
#   CONFIGURED_FPNAME="$FPUINAME"
#fi

FILEPACKE_NAME="TWS Client Install for"
TYPE_EP="Endpoint"
TYPE_MN="ManagedNode"

if [ "$IS_ENDPOINT" = false ]; then
    CONFIGURED_FPNAME="$FILEPACKE_NAME $TARGET $TYPE_MN"
else
    CONFIGURED_FPNAME="$FILEPACKE_NAME $TARGET $TYPE_EP"
fi
LINK_NAME="$CONFIGURED_FPNAME"

wcrtsprfl       -s "NCLINK"        \
                "$FPUINAME" \
                FilePackage \
                "$CONFIGURED_FPNAME" \
                "$FPPARENT" \
                "$LINK_NAME"

STATUS=$?
if [ $STATUS -ne 0 ] ; then
	errexit $ERRCONFIG "'"$CONFIGURED_FPNAME"'"
fi

wimprtfp -h @ManagedNode:"$SRC_HOST"  @FilePackage:"$CONFIGURED_FPNAME#$IRONAME" < $NEWFPDESC
STATUS=$?

if [ $STATUS -ne 0 ] ; then
	errmsg $ERRCONFIG "'"$CONFIGURED_FPNAME"'"
else
	echo $SUCCESSCONFIG "'"$CONFIGURED_FPNAME"'"
###	rm $NEWFPDESC
	echo $NEWFPDESC
fi

exit $STATUS

