#!/bin/sh
#
# File: 	config_nontme_logadapter.sh
#
# Product:	Tivoli Module
# Description:	Shell Script
# Author:		Module Generator
#
# Generated:	
#
# (C) COPYRIGHT Tivoli Systems, Inc., 1997
# All Rights Reserved
# Licensed Material - Property of Tivoli Systems, Inc.
#

###################
# Support Functions
###################

###
#
# restart_tecad
#
restart_tecad()
{
	if [ "$RESTART_TECAD" = "on" -o "$1" = "FORCE" ] ;then
		echo "Starting the TEC Logfile Adapter"
		if [ x\"$OS\" = x\"Windows_NT\" ] ; then
			$TECAD_START_CMD > /dev/null 2>& 1 
		else
			$TECAD_START_CMD > /dev/null 2>& 1 &
		fi
	fi
}

###
#
# backout
#
backout()
{
	errecho "backing out. Files = $*"
	eval `set $*`
	while [ -n "$*" ] 
	do
		##cp "$2" "$1"
		##rm "$2"
		eval a="$1"
		eval b="$2"
		eval cp \"$b\" \"$a\"
		eval rm \"$b\"
		shift 2
	done
	[ "$STOPPED_TECAD" = true ] && restart_tecad FORCE
	exit 1
}

###
#
# errexit
#
errexit()
{
	echo "$*" >&2
	exit 1
}


###
#
# errecho
#
errecho() {
  echo "$*" >&2
}

debugmsg()
{
	echo "$*"
}


###
#
# is_lcf_ep
#
is_lcf_ep() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if [ -z "$ENDPOINT_OID" ]; then
    return 1
  elif echo $ENDPOINT_OID | awk -F\# '{print $1}' | grep "+$" > /dev/null; then
    # found a "+" in the oid, must be a TMA endpoint!
    return 0
  else
    return 1
  fi
}

###
#
# is_managed_node
#
is_managed_node() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if [ -z "$ENDPOINT_OID" ]; then
    return 0
  elif echo $ENDPOINT_OID | awk -F\# '{print $1}' | grep "+$" > /dev/null; then
    # found a "+" in the oid, must be a TMA endpoint!
    return 1
  else
    return 0
  fi
}

###
#
# is_OS_NT
#
is_OS_NT() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if [ "`uname`" = "Windows_NT" ]; then
    return 0
  else
    return 1
  fi
}


###
#
#   Return value for BINDIR
#
bindir() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then
    OSERV=`objcall 0.0.0 get_oserv`; export OSERV
    INTERP=`objcall $OSERV query interp`; export INTERP
    inst_dir="`objcall $OSERV query install_dir`"
    bindir="$inst_dir/$INTERP"
  else
    bindir="$LCF_BINDIR/.."
  fi
  bindir="`echo $bindir | sed -e 's,\\\\,/,g'`"
  echo $bindir
}

###
#
#   Return value for INST_DIR
#
inst_dir() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then
    OSERV=`objcall 0.0.0 get_oserv`; export OSERV
    inst_dir="`objcall $OSERV query install_dir`"
  else
    inst_dir="$LCFROOT"
  fi
  inst_dir="`echo $inst_dir | sed -e 's,\\\\,/,g'`"
  echo $inst_dir
}

###
#
#   Return correct value for tmp directory
#
temp() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then
    temp=/tmp
    if is_OS_NT; then
	temp="$DBDIR/tmp"
    fi
  else
    if [ -n "$LCF_TEMPDIR" ]; then
      temp="$LCF_TEMPDIR"
    else
      temp="$LCF_DATDIR/tmp"
    fi
  fi
  if [ ! -d "$temp" ]; then
    mkdir -p "$temp"
  fi
  echo $temp
}

###
#
#    Find the Tivoli etc directory and source the right environment setup file
#
source_etc_setup_env() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then
    ETC=/etc/Tivoli

    # If this is NT, then the Tivoli etc directory is in a different location
    if is_OS_NT; then
       ETC=$SystemRoot/system32/drivers/etc/Tivoli
    fi

    # Look for the optional Plus override script, if not there use the standard one
    SETUP_SCRIPT=""
    if [ ! -f "$ETC/$SETUP_SCRIPT" ]; then
	SETUP_SCRIPT=setup_env.sh
    fi
    . $ETC/$SETUP_SCRIPT
  else
    # not a managed node; find the most recent lcf_env.sh and source it
    ETC=/etc/Tivoli/lcf

    # If this is NT, then the Tivoli etc directory is in a different location
    if is_OS_NT; then
       ETC=$SystemRoot/Tivoli/lcf
    fi
    SETUP_SCRIPT=lcf_env.sh
    LCF_ETC_SCRIPT=`echo \`ls -td ${ETC}/*\` | awk '{print $1}'`/$SETUP_SCRIPT
    if [ -f "$LCF_ETC_SCRIPT" ]; then
      . $LCF_ETC_SCRIPT
    fi
  fi
}

###
#
# ironame
#
ironame() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then
    #
    #	Get the region name to avoid collisions
    #
    iro=`wlookup InterRegion`
    ironame=`idlattr -t -g $iro name string`
    ironame=`eval echo $ironame`
    echo $ironame
  else
    echo ""
  fi
}

###
#
# mnname
#
mnname() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then
    mnoid=`objcall 0.0.0 get_host_location`
    eval mnname=`idlattr -t -g $mnoid label string`
    echo $mnname
  else
    echo ""
  fi
}

###
#
# source_product_info
#
source_product_info() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if [ -d "$1" ]; then
    cd "$1"
    if [ -f PLUSproduct-info.sh ]; then
      . ./PLUSproduct-info.sh
      export WINSTRUCTED
      export MODULE_VERSION
    else
      errecho "$1/PLUSproduct-info.sh not found!"
    fi
  else
    errecho "$1/PLUSproduct-info.sh not found!"
  fi
}

###
#
# get_install_opts
#
get_install_opts() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if [ -n "$INSTALL_OPTS" ]; then
   if [ -f "$LINKDIR/PLUSget_install_opts.sh" ]; then
	. "$LINKDIR/PLUSget_install_opts.sh"
   fi
  fi
}

###
#
# source_tws_plus_env
#

source_tws_plus_env()
{
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then

    TWS_PLUS_SCRIPT="tws_plus.sh"
    if [ -f "$ETC/$TWS_PLUS_SCRIPT" ]; then
      . $ETC/$TWS_PLUS_SCRIPT
    else
      MAESTROUSER=`wgetattr TWUSER @Presentation:"Product_Install (TWS)"`
      MAESTRODIR=`wgetattr TWSDIR @Presentation:"Product_Install (TWS)"`
      JSCONSOLEDIR=`wgetattr JSCONSOLEDIR @Presentation:"Product_Install (TWS)"`
      export MAESTROUSER MAESTRODIR JSCONSOLEDIR
    fi
  else
    TWS_PLUS_SCRIPT="tws_plus.sh"
    if [ -f "$LCF_DATDIR/$TWS_PLUS_SCRIPT" ]; then
      . "$LCF_DATDIR/$TWS_PLUS_SCRIPT"
    fi
  fi
}

##################################################################################
#
#
if [ -n "" ]; then
  set -x
  DEBUG=true
fi

TASK_PROGPATH=`echo $0 | sed -e 's.\\\\./.g'`
THIS_SCRIPT="config_nontme_logadapter.sh"

#
#	Set up some popular environment variables
#
source_etc_setup_env
BINDIR="`bindir`"; export BINDIR
INST_DIR="`inst_dir`"; export INST_DIR
MNNAME="`mnname`"; export MNNAME
TEMP="`temp`"; export TEMP
LCF_CACHEDIR="`echo $LCF_CACHEDIR | sed -e 's,\\\\,/,g'`"

#
#	Debugging code: by creating a directory called .plusdebug in
#		the appropriate TEMP directory, the output of scripts may be
#		redirected to a file in TEMP by the same name as the script.
#
DEBUGDIR="$TEMP/.plusdebug"
[ -d "$DEBUGDIR" ] \
  && echo "*** Results redirected to $DEBUGDIR/$THIS_SCRIPT" \
  && exec > "$DEBUGDIR/$THIS_SCRIPT" 2>&1 && set -xv
[ -f "$DEBUGDIR" ] && set -xv
[ "$DEBUG" = true ] && set -xv

#
#	Set up variables for our basic directories, path
#
PLUSDIR="$INST_DIR/generic_unix/TME/PLUS"
LINKDIR="$PLUSDIR/LINK"
PRODDIR="$PLUSDIR/TWS"
PATH="$LINKDIR:$PRODDIR:$PATH"
export PATH

#
#	Source the PLUSproduct-info.sh file
#
#source_product_info "$PRODDIR"

#
#	Source the Maestro Plus environment file
#
source_tws_plus_env

#
#	Messages
#
ERRNOFMTFILE="Error: Invalid value for argument \"Format File Pathname\""

FMT_FILE_HEAD="// TWS Event Logfile"
MODULE_TAG="MAESTRO"
SOURCE_NAME=PLUSTASKS

###################
# Process Arguments
###################

#
#	Usage Check
#
ARGLIST="Adapter Directory"

if [ $# -ne 1 ]; then
	errexit "Usage: $THIS_SCRIPT $ARGLIST"
fi

#EVENT_FILE="`echo $1 | sed -e 's,\\\\,/,g'`"

NONTME_ADAPTER_DIR="`echo $1 | sed -e 's,\\\\,/,g'`"

#"save (y)" or "abort (n)"
#EXIST_OPTION=$2
EXIST_OPTION="y"

#"on" or "off"
#RESTART_TECAD=$3
RESTART_TECAD="y"

#
# set up a list of paths to locate adapter files
#   NOTE: You may wish to modify this list to reflect other possible installations
#
#dirs_to_check='
#  "${LCF_CACHEDIR}/bin/${INTERP}/TME/TEC/adapters"
#  "$BINDIR/TME/TEC/adapters"
#  "$BINDIR/../../lcf/TME/TEC/adapters"
#'
dirs_to_check='"$NONTME_ADAPTER_DIR"'

for d in $dirs_to_check; do
  eval TECAD_DIR="$d"

  #
  # check the normal TECAD_DIR/bin first for the executable
  #
  TECAD_BINDIR="$TECAD_DIR/bin"
  if [ x\"$OS\" = x\"Windows_NT\" ] ; then
    if [ -f "$TECAD_BINDIR/tecad_nt.exe" ]; then
		TECAD_EXE="$TECAD_BINDIR/tecad_nt.exe"
    elif [ -f "$TECAD_BINDIR/tecad_win.exe" ]; then
		TECAD_EXE="$TECAD_BINDIR/tecad_win.exe"
	fi
  else
	TECAD_EXE="$TECAD_BINDIR/tecad_logfile"
  fi
  if [ -f "$TECAD_EXE" ]; then
	break
  fi

  #
  # some installs may have the executable in TECAD_DIR/etc; check there now!
  #
  TECAD_BINDIR="$TECAD_DIR/etc"
  if [ x\"$OS\" = x\"Windows_NT\" ] ; then
    if [ -f "$TECAD_BINDIR/tecad_nt.exe" ]; then
		TECAD_EXE="$TECAD_BINDIR/tecad_nt.exe"
    else
		TECAD_EXE="$TECAD_BINDIR/tecad_win.exe"
	fi
  else
	TECAD_EXE="$TECAD_BINDIR/tecad_logfile"
  fi
  if [ -f "$TECAD_EXE" ]; then
	break
  fi
done

for d in $dirs_to_check; do
  eval TECAD_DIR="$d"
  if [ x\"$OS\" = x\"Windows_NT\" ] ; then
    if [ -f "$TECAD_DIR/bin/nt_gencds.exe" ]; then
		TECAD_GENCDS_EXE="$TECAD_DIR/bin/nt_gencds.exe"
	else
		TECAD_GENCDS_EXE="$TECAD_DIR/bin/win_gencds.exe"
	fi
  else
	TECAD_GENCDS_EXE="$TECAD_DIR/bin/logfile_gencds"
  fi
  if [ -f "$TECAD_GENCDS_EXE" ]; then
	break
  fi
done



for d in $dirs_to_check; do
  eval TECAD_DIR="$d"
  TECAD_ETCDIR="$TECAD_DIR/etc"
  if [ -d "$TECAD_ETCDIR" ]; then
	break
  fi
done

[ ! -d "$TECAD_BINDIR" ] &&  echo "This node does not have the TEC logfile bin directory installed" >&2 && exit 1
[ ! -d "$TECAD_ETCDIR" ] &&  echo "This node does not have the TEC logfile etc directory installed" >&2 && exit 1
[ ! -x "$TECAD_EXE" ] &&  echo "This node does not have the TEC logfile adapter installed" >&2 && exit 1
[ ! -x "$TECAD_GENCDS_EXE" ] &&  echo "This node does not have the TEC logfile gencds command installed" >&2 && exit 1


TWS_FMT_FILE=maestro.fmt
if is_OS_NT; then
    TWS_FMT_FILE=maestront.fmt
fi
export TWS_FMT_FILE


if is_lcf_ep; then
    TEC_FMT_FILE_1="$LCF_DATDIR/$TWS_FMT_FILE"
else
    TEC_FMT_FILE_1="$DBDIR/$TWS_FMT_FILE"
fi
if is_lcf_ep; then
  LOCPATH="$LCF_DATDIR/PLUS"
else
  LOCPATH="$BINDIR/bin"
fi

TEC_FMT_FILES="TEC_FMT_FILE_1"

if [ ! -f "$TEC_FMT_FILE_1" ]; then
    #
    # try to see if this is a managed node where module is installed
    # 
    ETC=/etc/Tivoli
    if is_OS_NT; then
	ETC=$SystemRoot/system32/drivers/etc/Tivoli
    fi
   
    fmtfile=`(
	if [ -f $ETC/setup_env.sh ]; then
	    . $ETC/setup_env.sh
	    if [ -f "$BINDIR/../generic_unix/TME/PLUS/TWS/$TWS_FMT_FILE" ]; then
		echo "$BINDIR/../generic_unix/TME/PLUS/TWS/$TWS_FMT_FILE"
	    else
		echo ""
	    fi
	else
	    echo ""
	fi
    )`
   
    TEC_FMT_FILE_1=$fmtfile
fi

if [ ! -f "$TEC_FMT_FILE_1" ]; then
   if is_lcf_ep; then
    fmtfile=`(
	 if [ -f "$LCF_DATDIR/PLUS/$TWS_FMT_FILE" ]; then
	     cp "$LCF_DATDIR/PLUS/$TWS_FMT_FILE" "$TECAD_ETCDIR"
	     echo "$TECAD_ETCDIR/$TWS_FMT_FILE"
	 else
	     echo ""
	 fi
    )`
   else
       DISPATCHER=`objcall 0.0.0 self | awk -F. '{print $2}'`
       REGION=`objcall 0.0.0 self | awk -F. '{print $1}'`

       TMR_MN=`wlookup -ar ManagedNode | grep "$REGION\.1\." | awk '{print $2}'`
       TMR_OSERV=`objcall $TMR_MN get_oserv`
       TMR_BINDIR=`objcall $TMR_OSERV query install_dir | tr "\\\\\\\\" "/"`
       TPLUS_DIR="$TMR_BINDIR/generic_unix/TME/PLUS/TWS"
       MN=`wlookup -ar ManagedNode | grep $REGION.$DISPATCHER | awk '{print $2}'`
`idlcall $TMR_MN files_transfer FALSE $MN \"$TPLUS_DIR\" \"$TECAD_ETCDIR\" \{ 1 \"$TWS_FMT_FILE\" \}`
       fmtfile=`(
	 if [ -f "$TECAD_ETCDIR/$TWS_FMT_FILE" ]; then
	     echo "$TECAD_ETCDIR/$TWS_FMT_FILE"
	 else
	     echo ""
	 fi
       )`
   fi
   TEC_FMT_FILE_1=$fmtfile
fi


if [ ! -f "$TEC_FMT_FILE_1" ]; then
	errexit "$ERRNOFMTFILE" "$TEC_FMT_FILE_1"
fi
#
#	check incoming arguments for validity
#
#if [ ! "$EVENT_FILE" ]; then
#	errexit "$ERRNOEVENTLOG"
#fi
for file in $TEC_FMT_FILES; do
	eval FMTFILENAME='"$'$file'"'
	if [ ! -f "$FMTFILENAME" ]; then
		errexit "$ERRNOFMTFILE"
	elif grep FORMAT "$FMTFILENAME" > /dev/null 2>&1 ; then
		:
	else
		errexit "Error: $FMTFILENAME is not a valid format file"
	fi
done

[ "$RESTART_TECAD" = "y" ] && RESTART_TECAD="on"

FMTAD_PATH=$TECAD_ETCDIR
TMP_ADPATH=`"$LOCPATH/wlocpath" $TECAD_ETCDIR/%L -d $TECAD_ETCDIR/C`
if [ -d "$TMP_ADPATH" ] ; then
	FMTAD_PATH=$TMP_ADPATH
fi


if [ x\"$OS\" = x\"Windows_NT\" ] ; then
	if [ -f "$TECAD_ETCDIR/tecad_nt.conf" ]; then
		TECAD_CONF="$TECAD_ETCDIR/tecad_nt.conf"
	else
		TECAD_CONF="$TECAD_ETCDIR/tecad_win.conf"
	fi
    if [ -f "$FMTAD_PATH/tecad_nt.fmt" ]; then
		TECAD_FMT="$FMTAD_PATH/tecad_nt.fmt"
	else
		TECAD_FMT="$FMTAD_PATH/tecad_win.fmt"
	fi		
    if [ -f "$TECAD_ETCDIR/tecad_nt.cds" ]; then
		TECAD_CDS="$TECAD_ETCDIR/tecad_nt.cds"
	else
		TECAD_CDS="$TECAD_ETCDIR/tecad_win.cds"
	fi
	if [ x\"$TECAD_EXE\" = x\"$TECAD_BINDIR/tecad_win.exe\" ] ; then
		TECAD_START_CMD="net start TECWINadapter"
		TECAD_STOP_CMD="net stop TECWINadapter"
	else
		TECAD_START_CMD="net start TECNTadapter"
		TECAD_STOP_CMD="net stop TECNTadapter"
	fi
else
	TECAD_CONF="$TECAD_ETCDIR/tecad_logfile.conf"
	TECAD_FMT="$FMTAD_PATH/tecad_logfile.fmt"
	TECAD_CDS="$TECAD_ETCDIR/tecad_logfile.cds"
	TECAD_START_CMD="$TECAD_BINDIR/init.tecad_logfile start"
	TECAD_STOP_CMD="$TECAD_BINDIR/init.tecad_logfile stop"
fi


EVENT_FILE=$MAESTRODIR/event.log

#Copy the file BmEvents.conf from the $MAESTRODIR/OV directory into
#MAESTRODIR
if [ ! -f "$MAESTRODIR/BmEvents.conf" ] ; then
	echo "$MAESTRODIR/BmEvents.conf does not exist, searching..."
	#We need to locate a BmEvents.conf file to copy to MAESTRODIR
	if [ -f "$MAESTRODIR/OV/BmEvents.conf" ] ; then
		COPYFILE="$MAESTRODIR/OV/BmEvents.conf"
	else
		if [ -f "$MAESTRODIR/unsupported/OpC/BmEvents.conf" ] ; then
			COPYFILE="$MAESTRODIR/unsupported/OpC/BmEvents.conf"
		else
			if [ -f "$MAESTRODIR/config/BmEvents.conf" ] ; then
				COPYFILE="$MAESTRODIR/config/BmEvents.conf"
			fi
		fi
	fi
	if [ ! -z "$COPYFILE" ] ; then
		echo "Found $COPYFILE."
		cp "$COPYFILE" "$MAESTRODIR"
		if [ $? -ne 0 ] ; then
			errecho "Unable to copy $COPYFILE to $MAESTRODIR"
			exit 1
		else
			echo "Copied $COPYFILE to $MAESTRODIR"
		fi
		#Make sure that BmEvents.conf is readable
		if [ x\"$OS\" != x\"Windows_NT\" ] ; then
			chmod +r "$MAESTRODIR/BmEvents.conf" > /dev/null 2>&1
			chown $MAESTROUSER "$MAESTRODIR/BmEvents.conf" > /dev/null 2>&1
		else
			$MAESTRODIR/bin/setown.exe -u "$MAESTROUSER" "$MAESTRODIR/BmEvents.conf" > /dev/null 2>&1
		fi
	else
		#Bad news we don't have a BmEvents.conf to snatch
		errecho "Could not find a BmEvents.conf file to copy."
		exit 1 
	fi

#	if [ x\"$OS\" != x\"Windows_NT\" ] ; then
		echo "Configuring Maestro to send events to $MAESTRODIR/event.log."
	    sed '/FILE/d' "$MAESTRODIR/BmEvents.conf" >> "$MAESTRODIR/BmEvents.conf.tmp"
	    echo "FILE=$MAESTRODIR/event.log" >> "$MAESTRODIR/BmEvents.conf.tmp"
	    mv "$MAESTRODIR/BmEvents.conf.tmp" "$MAESTRODIR/BmEvents.conf"
#	fi

else
	HAS_LOGFILE=`grep '^[	]*FILE[ 	]*=' "$MAESTRODIR/BmEvents.conf" 2> /dev/null`

	if [ -z "$HAS_LOGFILE" ] ; then 
		#We need to add a FILE= directive to $MAESTRODIR/BmEvents.conf
		echo "Configuring Maestro to send events to $MAESTRODIR/event.log."
		echo "FILE=$MAESTRODIR/event.log" >> "$MAESTRODIR/BmEvents.conf"
		LOGFILE="$MAESTRODIR/event.log"
	else
		#Maestro is already configured to send events to a file
		#See which one it is
		LOGFILE=`echo $HAS_LOGFILE | awk -F= '{print $NF}'`
		echo "Maestro logfile already set to $LOGFILE"
	fi
fi

PATH=/bin:/usr/bin:/usr/lib:${MAESTRODIR}:${MAESTRODIR}/bin:${PATH}
export PATH

#Check BmEvents.conf to see if there is already an EVENTS= directive
HAS_EVENTS=`grep '^[	]*EVENTS[	]*=' "$MAESTRODIR/BmEvents.conf" 2> /dev/null`

if [ -z "$HAS_EVENTS" ] ; then 
	#We need to add an EVENTS= directive to $MAESTRODIR/BmEvents.conf
	echo "Configuring Maestro to report all events"
	echo "/EVENTS/a\\" > "$TEMP/sed.command"
	echo "EVENTS=1 51 52 53 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 151 152 153 154 155 156 157 158 159 160 161 162 201 202 203 204 251 252 301" >> "$TEMP/sed.command"
	sed -f "$TEMP/sed.command" "$MAESTRODIR/BmEvents.conf" > "$TEMP/BmEvents.conf"

	HAS_EVENTS=`grep '^[	]*EVENTS[	]*=' "$TEMP/BmEvents.conf" 2> /dev/null`

	if [ -z "$HAS_EVENTS" ] ; then
		#A problem occurred running the sed command.
		echo "Unable to add the EVENTS directive to $MAESTRODIR/BmEvents.conf"
		echo "The default Maestro events will be sent to $LOGFILE"
		echo "The $MAESTRODIR/BmEvents.conf file must be edited manually to include more events."
	else
		#Move the modified BmEvents.conf into the maestro dir.
		cp "$TEMP/BmEvents.conf" "$MAESTRODIR/BmEvents.conf"
	fi
else
	 echo "Maestro already configured to send selected events"
fi

#Get File directive and added t adapter conf
GET_LOGFILE=`grep '^[	]*FILE[ 	]*=' "$MAESTRODIR/BmEvents.conf" 2> /dev/null`
EV_FILE=`echo $GET_LOGFILE | awk -F= '{print $NF}'`
EVENT_FILE="`echo $EV_FILE | sed -e 's,\\\\,/,g'`"

for file in `sed '/[	]*LogSources[	]*=/ {
   s/[	]*#$//
   s/,/ /g
   s/[	]*LogSources[	]*=//
   p
}
{
   d
}' "$TECAD_CONF"` ;do

	if [ "$file" = "$EVENT_FILE" ] ;then
		EXISTING_EVENT_FILE="$file"
	fi
done
 
if [ ! -z "$EXISTING_EVENT_FILE" ] ;then
	LOGFILE_IN_TECAD=`grep "$FMT_FILE_HEAD" "$TECAD_FMT" 2> /dev/null`
fi

if [ -z "$EXISTING_EVENT_FILE" -o -z "$LOGFILE_IN_TECAD" ] ;then
	# if the files are not set up properly force configuration anyway
	FORCE_CONFIGURE=1
else
	if [ "$EXIST_OPTION" = "abort" -o "$EXIST_OPTION" != "y" ] ;then
		echo "Existing Files Found. Skipping Configuration on this node."
		restart_tecad
		exit 0
	fi
fi

#
# if we got this far, we really want to configure 
#

if [ "$RESTART_TECAD" = "on" ] ;then
	echo "Stopping the TEC Logfile Adapter"
	$TECAD_STOP_CMD
	STOPPED_TECAD=true
fi

[ -f "$TECAD_CONF" ] && \
	cp "$TECAD_CONF" "$TECAD_CONF.pre.$MODULE_TAG" && \
	BACKOUTLIST="$BACKOUTLIST '"'$TECAD_CONF'"' '"'$TECAD_CONF.pre.$MODULE_TAG'"'"

if [ -z "$EXISTING_EVENT_FILE" ] ; then
	SOURCES="$EVENT_FILE"
fi

if l=`grep '^[	]*LogSources[	]*=' "$TECAD_CONF" 2>/dev/null`; then
	echo "$l" | grep "$SOURCES" >/dev/null 2>&1 ||
	sed '/^[	]*LogSources[	]*=/	{
		s;=\([	]*\);='$SOURCES',\1;
		s;,[	]*$;;
		s;,\([	]*#\);\1;
	}' "$TECAD_CONF.pre.$MODULE_TAG" > "$TECAD_CONF"
else
	sed '$d' "$TECAD_CONF" > "${TECAD_CONF}.tmp"
	echo "LogSources=$SOURCES" >> "${TECAD_CONF}.tmp"
	sed -n '$p' "$TECAD_CONF" >> "${TECAD_CONF}.tmp"
	mv "${TECAD_CONF}.tmp" "$TECAD_CONF"
fi

echo "Added LogSources line with $SOURCES to $TECAD_CONF"

if l=`grep '^[	]*PollInterval[ 	]*=' "$TECAD_CONF" 2>/dev/null`; then
	echo "PollInterval was already set"
else 
	echo "PollInterval=10" >> "$TECAD_CONF"
	echo "Added PollInterval line to $TECAD_CONF"
fi

if [ $? -ne 0 ] ;then
	backout $BACKOUTLIST
fi

if [ -z "$LOGFILE_IN_TECAD" ] ; then
	# We need to cat fmt to the tecad_logfile.fmt file
	[ -f "$TECAD_FMT" ] && cp "$TECAD_FMT" "$TECAD_FMT.pre.$MODULE_TAG" && \
		BACKOUTLIST="$BACKOUTLIST '"'$TECAD_FMT'"' '"'$TECAD_FMT.pre.$MODULE_TAG'"'"
	[ -f "$TECAD_CDS" ] && cp "$TECAD_CDS" "$TECAD_CDS.pre.$MODULE_TAG" && \
		BACKOUTLIST="$BACKOUTLIST '"'$TECAD_CDS'"' '"'$TECAD_CDS.pre.$MODULE_TAG'"'"
	for file in $TEC_FMT_FILES; do
		eval FMTFILENAME='"$'$file'"'
		
		if grep FORMAT "$FMTFILENAME" > /dev/null 2>&1 ; then
			cat "$FMTFILENAME" >> "$TECAD_FMT"
		else
			echo "Error: $FMTFILENAME is not a valid format file" >&2
			backout $BACKOUTLIST
		fi
	done

	"$TECAD_GENCDS_EXE" "$TECAD_FMT" > "$TECAD_CDS"
	GENCDS_STATUS=$?
	if [ $GENCDS_STATUS = 0 ] ; then
		echo "Succesfully generated $TECAD_CDS file."
	else
		errecho "$TECAD_GENCDS_EXE exited with non-zero status [$GENCDS_STATUS]"
		backout $BACKOUTLIST
	fi
fi


echo "Node Configured Succesfully."

restart_tecad

exit 0
