#!/bin/sh
#
# File:		maestro_fp_post.sh
#
# Product:	Tivoli Module
# Description:	Shell Script
# Author:		Module Generator
#
# Generated:	
#
# (C) COPYRIGHT Tivoli Systems, Inc., 1997
# All Rights Reserved
# Licensed Material - Property of Tivoli Systems, Inc.
#

###
#
#   errecho
#
errecho() {
  echo "$*" >&2
}

###
#
#   errexit
#
errexit()
{
        echo "$*" >&2
        exit 1
}


###
#
# is_lcf_ep
#
is_lcf_ep() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if [ "$IS_ENDPOINT" = "true" ]; then
      return 0;
  else
      return 1;
  fi
#  if [ -z "$ENDPOINT_OID" ]; then
#    return 1
#  elif echo $ENDPOINT_OID | awk -F\# '{print $1}' | grep "+$" > /dev/null; then
#    # found a "+" in the oid, must be a TMA endpoint!
#    return 0
#  else
#    return 1
#  fi
}

###
#
# is_managed_node
#
is_managed_node() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if [ "$IS_ENDPOINT" = "false" ]; then
      return 0;
  else
      return 1;
  fi
#  if [ -z "$ENDPOINT_OID" ]; then
#    return 0
#  elif echo $ENDPOINT_OID | awk -F\# '{print $1}' | grep "+$" > /dev/null; then
#    # found a "+" in the oid, must be a TMA endpoint!
#    return 1
#  else
#    return 0
#  fi
}

###
#
# is_OS_NT
#
is_OS_NT() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if [ "`uname`" = "Windows_NT" ]; then
    return 0
  else
    return 1
  fi
}


###
#
#   Return value for BINDIR
#
bindir() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then
    OSERV=`objcall 0.0.0 get_oserv`; export OSERV
    INTERP=`objcall $OSERV query interp`; export INTERP
    inst_dir="`objcall $OSERV query install_dir`"
    bindir="$inst_dir/$INTERP"
  else
    bindir="$LCF_BINDIR/.."
  fi
  bindir="`echo $bindir | sed -e 's,\\\\,/,g'`"
  echo $bindir
}

###
#
#   Return value for INST_DIR
#
inst_dir() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then
    OSERV=`objcall 0.0.0 get_oserv`; export OSERV
    inst_dir="`objcall $OSERV query install_dir`"
  else
    inst_dir="$LCFROOT"
  fi
  inst_dir="`echo $inst_dir | sed -e 's,\\\\,/,g'`"
  echo $inst_dir
}

###
#
#   Return correct value for tmp directory
#
temp() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then
    temp=/tmp
    if is_OS_NT; then
	temp="$DBDIR/tmp"
    fi
  else
    if [ -n "$LCF_TEMPDIR" ]; then
      temp="$LCF_TEMPDIR"
    else
      temp="$LCF_DATDIR/tmp"
    fi
  fi
  if [ ! -d "$temp" ]; then
    mkdir -p "$temp"
  fi
  echo $temp
}

###
#
#    Find the Tivoli etc directory and source the right environment setup file
#
source_etc_setup_env() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then
    ETC=/etc/Tivoli

    # If this is NT, then the Tivoli etc directory is in a different location
    if is_OS_NT; then
       ETC=$SystemRoot/system32/drivers/etc/Tivoli
    fi

    # Look for the optional Plus override script, if not there use the standard one
    SETUP_SCRIPT=""
    if [ ! -f "$ETC/$SETUP_SCRIPT" ]; then
	SETUP_SCRIPT=setup_env.sh
    fi
    . $ETC/$SETUP_SCRIPT
  else
    # not a managed node; find the most recent lcf_env.sh and source it
    ETC=/etc/Tivoli/lcf

    # If this is NT, then the Tivoli etc directory is in a different location
    if is_OS_NT; then
       ETC=$SystemRoot/Tivoli/lcf
    fi
    SETUP_SCRIPT=lcf_env.sh
    LCF_ETC_SCRIPT=`echo \`ls -td ${ETC}/*\` | awk '{print $1}'`/$SETUP_SCRIPT
    if [ -f "$LCF_ETC_SCRIPT" ]; then
      . $LCF_ETC_SCRIPT
    fi
  fi
}

###
#
# ironame
#
ironame() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then
    #
    #	Get the region name to avoid collisions
    #
    iro=`wlookup InterRegion`
    ironame=`idlattr -t -g $iro name string`
    ironame=`eval echo $ironame`
    echo $ironame
  else
    echo ""
  fi
}

###
#
# mnname
#
mnname() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if is_managed_node; then
    mnoid=`objcall 0.0.0 get_host_location`
    eval mnname=`idlattr -t -g $mnoid label string`
    echo $mnname
  else
    echo ""
  fi
}

###
#
# source_product_info
#
source_product_info() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if [ -d "$1" ]; then
    cd "$1"
    if [ -f PLUSproduct-info.sh ]; then
      . ./PLUSproduct-info.sh
      export WINSTRUCTED
      export MODULE_VERSION
    else
      errecho "$1/PLUSproduct-info.sh not found!"
    fi
  else
    errecho "$1/PLUSproduct-info.sh not found!"
  fi
}

###
#
# get_install_opts
#
get_install_opts() {
  if [ "$DEBUG" = true ]; then set -x; fi
  if [ -n "$INSTALL_OPTS" ]; then
   if [ -f "$LINKDIR/PLUSget_install_opts.sh" ]; then
	. "$LINKDIR/PLUSget_install_opts.sh"
   fi
  fi
}

##################################################################################
#
#
if [ -n "" ]; then
  set -x
  DEBUG=true
fi

TASK_PROGPATH="`echo $0 | sed -e 's.\\\\./.g'`"
THIS_SCRIPT="maestro_fp_post.sh"

#
#	Set up some popular environment variables
#
source_etc_setup_env
BINDIR="`bindir`"; export BINDIR
INST_DIR="`inst_dir`"; export INST_DIR
MNNAME="`mnname`"; export MNNAME
TEMP="`temp`"; export TEMP

#
#	Debugging code: by creating a directory called .plusdebug in
#		the appropriate TEMP directory, the output of scripts may be
#		redirected to a file in TEMP by the same name as the script.
#
DEBUGDIR="$TEMP/.plusdebug"
[ -d "$DEBUGDIR" ] \
  && echo "*** Results redirected to $DEBUGDIR/$THIS_SCRIPT" \
  && exec > "$DEBUGDIR/$THIS_SCRIPT" 2>&1 && set -xv
[ -f "$DEBUGDIR" ] && set -xv
[ "$DEBUG" = true ] && set -xv

#
#	Set up variables for our basic directories, path
#
PLUSDIR="$INST_DIR/generic_unix/TME/PLUS"
LINKDIR="$PLUSDIR/LINK"
PRODDIR="$PLUSDIR/TWS"
PATH="$LINKDIR:$PRODDIR:$PATH"
export PATH

if [ -z "$IS_ENDPOINT" ] ; then
	error_msg="Must set IS_ENDPOINT environment variable"
        errexit $error_msg
fi

if [ -z "$MAESTRO_DIR" ] ; then
	error_msg="Must set MAESTRO_DIR environment variable"
	errexit $error_msg
fi

if [ -z "$MAESTRO_USER" ] ; then
	error_msg="Must set MAESTRO_USER environment variable"
	errexit $error_msg
fi


TWS_PLUS_SH=tws_plus.sh
    
if is_managed_node; then
    SCRIPT_DIR=$ETC
else
    SCRIPT_DIR=$LCF_DATDIR
fi

maestro_user=$MAESTRO_USER
maestro_dir="`echo $MAESTRO_DIR | sed -e 's,\\\\,/,g'`"

if [ -n "$SCRIPT_DIR" ]; then
    
    TWS_PLUS_SH=tws_plus.sh
    MAESTRO_PLUS_CSH=maestro_plus.csh
    
    echo "MAESTROUSER=$maestro_user" > $SCRIPT_DIR/$TWS_PLUS_SH
    echo "MAESTRODIR=$maestro_dir" >> $SCRIPT_DIR/$TWS_PLUS_SH
    echo "export MAESTROUSER MAESTRODIR" >> $SCRIPT_DIR/$TWS_PLUS_SH

fi


NODENAME=`uname -n | tr [A-Z] [a-z]`
export NODENAME

PATH=/tmp/.maestro_tme:/bin:/usr/bin:/usr/sbin:/usr/ucb:$PATH
export PATH
PATH=${MAESTRO_DIR}:${MAESTRO_DIR}/bin:${PATH}
export PATH

if [ -z "$MAP_FILE" ] ; then
	error_msg="Must set MAP_FILE environment variable"
	errexit $error_msg
fi

MAP_NAME_ONLY=`echo $MAP_FILE | awk -F/ '{print $NF}'`

if [ ! -f /tmp/.maestro_tme/$MAP_NAME_ONLY ] ; then
	error_msg="Could not find the TWS cpu map file named \"/tmp/.maestro_tme/$MAP_NAME_ONLY\""
	errecho $error_msg
	exit 2
fi

HAS_KEY=`grep "^$NODENAME:" /tmp/.maestro_tme/$MAP_NAME_ONLY`

if [ -z "$HAS_KEY" ] ; then
	error_msg="There is no entry for host \"$NODENAME\" in the TWS cpu map \"/tmp/.maestro_tme/$MAP_NAME_ONLY\""
        errecho $error_msg
	exit 3
fi

#We found a map entry with the desired name
NODENAME=`echo $HAS_KEY | awk -F: '{print $1}'`
THIS_CPUNAME=`echo $HAS_KEY | awk -F: '{print $2}'`
CPU_TYPE=`echo $HAS_KEY | awk -F: '{print $3}'`
MASTER_NODENAME=`echo $HAS_KEY | awk -F: '{print $4}'`
MASTER_CPUNAME=`echo $HAS_KEY | awk -F: '{print $5}'`

#Check to see if the fields are all set correctly

#Check THIS_CPUNAME
if [ -z "$THIS_CPUNAME" ] ; then
	error_msg="Must have a cpuname for node \"$NODENAME\" in the TWS cpu map for this file package"
	errecho $error_msg
	exit 4
fi


#Check to see if the value for CPU_TYPE is a known type before attempting
#the customize script

case "$CPU_TYPE" in
	master)
		if [ x"$MASTER_NODENAME" = x ] ; then
			MASTER_NODENAME="$NODENAME"
			MASTER_CPUNAME="$THIS_CPUNAME"
		fi

		if [ x"$NODENAME" != x"$MASTER_NODENAME" ] ; then
			error_msg="node \"$NODENAME\" and master_node \"$MASTER_NODENAME\" must match in order to install this host as a TWS master"
                        errecho $error_msg
			exit 15
		fi
 

		if [ x"$THIS_CPUNAME" != x"$MASTER_CPUNAME" ] ; then
			error_msg="cpuname \"$THIS_CPUNAME\" and master_cpuname \"$MASTER_CPUNAME\" must match in order to install this cpu as a TWS master"
			errecho $error_msg
			exit 20
		fi
		;;
	fta)
		;;
	sagent)
		;;
	backup)
		;;
	*)
		error_msg="Invalid type \"$CPU_TYPE\" specified for cpu name \"$THIS_CPUNAME\". Valid types are master, fta, sagent, backup"
 		errecho $error_msg
		exit 10
		;;
esac

if [ -z "$MASTER_NODENAME" ] ; then 
        error_msg="Must supply a master_node name for node \"$NODENAME\" in TWS cpu map file"
	errecho $error_msg
        exit 12
fi

if [ -z "$MASTER_CPUNAME" ] ; then 
        error_msg="Must supply a master_cpuname name for node \"$NODENAME\" in TWS cpu map file"
	errecho $error_msg
        exit 14
fi

echo Correct parameters are available to the afterscript >&2
#Clean up /usr/bin
#cd /$MAESTRO_DIR/../bin
#/bin/rm datecalc jobstdl maestro at batch morestdl parms > /dev/null 2>&1
cd $MAESTRO_DIR

echo "Running customize script in directory \"$MAESTRO_DIR\" with command:" >&2
echo "/bin/sh ./customize -new -thiscpu \"$THIS_CPUNAME\" -master \"$MASTER_CPUNAME\" -uname \"$MAESTRO_USER\"" >&2

/bin/sh ./customize -new -thiscpu "$THIS_CPUNAME" -master "$MASTER_CPUNAME" -uname "$MAESTRO_USER" > /tmp/customize.log 2>&1

STATUS=$?

if [ $STATUS -ne 0 ] ;then
	echo "Customize failed, exit status = $STATUS" >&2
	echo "Check /tmp/customize.log on $NODENAME" >&2
	exit 25
fi


if [ ! -f $MAESTRO_DIR/../unison/Security ] ; then
	#security setup failed, probably because the evaluation key on the media
	#expired
	echo "Creating Security file, running command:" >&2
	echo "$MAESTRO_DIR/bin/makesec -l $MAESTRO_DIR/Security" >&2
	#So we'll try to run it by hand
	$MAESTRO_DIR/bin/makesec -l $MAESTRO_DIR/Security > /tmp/makesec.log 2>&1
	STATUS=$?

	if [ $STATUS -ne 0 ] ; then
		echo "makesec failed, exit status=$STATUS" >&2
		echo "Check /tmp/makesec.log on $NODENAME" >&2
		exit 30
	fi
		
fi

#Add the necessary cpus
case "$CPU_TYPE" in
	master)
		#Add local cpu
		echo "Adding the necessary local cpu definition" >&2
		maestro_add_cpu.sh "$IS_ENDPOINT" "$THIS_CPUNAME" "$NODENAME" unix master 
		;;
	fta)
		#Add local cpu
		#maestro_add_cpu.sh "$THIS_CPUNAME" "$NODENAME" unix fta 
		#Add master cpu
		#maestro_add_cpu.sh "$MASTER_CPUNAME" "$MASTER_NODENAME" unix master 
		;;
	sagent)
		#Add local cpu
		#maestro_add_cpu.sh "$THIS_CPUNAME" "$NODENAME" unix sagent "$MASTER_CPUNAME" 
		#Add master cpu
		#maestro_add_cpu.sh "$MASTER_CPUNAME" "$MASTER_NODENAME" unix master 
		
		;;
	backup)
		echo "Adding the necessary local cpu definitions" >&2
	#Add local cpu
		maestro_add_cpu.sh "$IS_ENDPOINT" "$THIS_CPUNAME" "$NODENAME" unix backup 
		#Add master cpu
		maestro_add_cpu.sh "$IS_ENDPOINT" "$MASTER_CPUNAME" "$MASTER_NODENAME" unix master 
		;;
esac

# run only on managed nodes
if [ "$IS_ENDPOINT" = "false" ]; then
    if [ "$CPU_TYPE" != "master" ] ; then

        IS_MASTER_ENDPOINT=true
        wlookup -r ManagedNode $MASTER_NODENAME > /dev/null 2>&1
        if [ $? -eq 0 ]; then
            IS_MASTER_ENDPOINT=false 
	fi
  
    	echo Adding the necessary cpu definition on master cpu >&2
    	if [ "$CPU_TYPE" != "sagent" ] ; then
    		#Add local cpu to the maestro master
    		wruntask -t "TWS_Add_CPU" -l "Hidden Sentry Tasks for TWS"  -h "$MASTER_NODENAME" -a "$IS_MASTER_ENDPOINT" -a "$THIS_CPUNAME" -a "$NODENAME" -a unix -a "$CPU_TYPE" -o 15 > /tmp/maestro_add_cpu.out 2>&1
     	else
    		#Add local cpu to the maestro master
    		wruntask -t "TWS_Add_CPU" -l "Hidden Sentry Tasks for TWS"  -h "$MASTER_NODENAME" -a "$IS_MASTER_ENDPOINT" -a "$THIS_CPUNAME" -a "$NODENAME" -a unix -a "$CPU_TYPE" -a "$MASTER_CPUNAME" -o 15 > /tmp/maestro_add_cpu.out 2>&1
    	fi
    fi
fi

#Now we need to add the startup lines to /etc/rc.local, /etc/rc3.d
#or /etc/rc
#We will build the command in a separate file so that it can be executed
#from either place
case "$CPU_TYPE" in
	master)
		cat > /tmp/maestro_control << TEXAS
#!/bin/sh
if [ -x $MAESTRO_DIR/bin/conman ] ; then
	echo "TWS started..."
	/bin/su - "$MAESTRO_USER" -c "$MAESTRO_DIR/bin/conman 'link @;noask&start @;noask'"
fi
TEXAS
		;;
	fta)
		cat > /tmp/maestro_control << TEXAS
#!/bin/sh
if [ -x $MAESTRO_DIR/bin/conman ] ; then
	echo "TWS started..."
	/bin/su - "$MAESTRO_USER" -c "$MAESTRO_DIR/bin/conman 'link $MASTER_CPUNAME;noask&start;noask'"
fi
TEXAS
		;;
	sagent)
		cat > /tmp/maestro_control << TEXAS
#!/bin/sh
if [ -x $MAESTRO_DIR/StartUp ] ; then
	echo "TWS started..."
	/bin/su - "$MAESTRO_USER" -c "$MAESTRO_DIR/bin/conman 'link $MASTER_CPUNAME;noask&start;noask'"
fi
TEXAS
		;;
	backup)
		cat > /tmp/maestro_control << TEXAS
#!/bin/sh
if [ -x $MAESTRO_DIR/StartUp ] ; then
	echo "netman started..."
	/bin/su - "$MAESTRO_USER" -c "$MAESTRO_DIR/StartUp"
fi
TEXAS
		;;
esac

chmod +x /tmp/maestro_control

#We need to either put the file we just created in /etc/rc3.d
#or add lines to start it from /etc/rc.local or /etc/rc
if [ -d /etc/rc3.d ] ; then
	cp /tmp/maestro_control /etc/rc3.d/S99maestro
	echo "Created /etc/rc3.d/S99maestro..." >&2
else
	cp /tmp/maestro_control /etc/maestro_start.sh

	if [ -f /etc/rc.local ] ; then
		FOUNDIT=`grep "#Start Maestro process" /etc/rc.local 2> /dev/null`
		if [ -z "$FOUNDIT" ] ; then
			cp /etc/rc.local /etc/rc.local.premaestro 
			echo "#Start Maestro process" >> /etc/rc.local
			echo "if [ -f /etc/maestro_start.sh ] ; then" >> /etc/rc.local
			echo "	/etc/maestro_start.sh" >> /etc/rc.local
			echo "fi" >> /etc/rc.local
			echo "Modified /etc/rc.local..." >&2
		fi

	elif [ -f /etc/rc.nfs ] ; then
		FOUNDIT=`grep "#Start Maestro process" /etc/rc.nfs 2> /dev/null`
		if [ -z "$FOUNDIT" ] ; then
			cp /etc/rc.nfs /etc/rc.nfs.premaestro 
			echo "#Start Maestro process" >> /etc/rc.nfs
			echo "if [ -f /etc/maestro_start.sh ] ; then" >> /etc/rc.nfs
			echo "	/etc/maestro_start.sh" >> /etc/rc.nfs
			echo "fi" >> /etc/rc.nfs
			echo "Modified /etc/rc.nfs..." >&2
		fi

	elif [ -f /etc/rc ] ;  then			
		FOUNDIT=`grep "#Start Maestro process" /etc/rc 2> /dev/null`
		if [ -z "$FOUNDIT" ] ; then 
			echo "/localrc\$/a\\" > /tmp/rc.mod
			echo "#Start Maestro process\\" >> /tmp/rc.mod
			echo "if [ -f /etc/maestro_start.sh ] ; then\\" >> /tmp/rc.mod
			echo "	/etc/maestro_start.sh\\" >> /tmp/rc.mod
			echo "fi" >> /tmp/rc.mod
			sed -f /tmp/rc.mod /etc/rc > /tmp/rc.new

			if [ $? -eq 0 ] ; then
				#It worked
				chmod 544 /tmp/rc.new
				mv /etc/rc /etc/rc.premaestro
				mv /tmp/rc.new /etc/rc
				echo "Modified /etc/rc..." >&2
			fi
		fi
	fi
fi

echo "Running StartUp command on $NODENAME..." >&2
/bin/su - "$MAESTRO_USER" -c "StartUp"

exit 0
